--- lib/kernel/src/net_kernel.erl.orig	2007-11-26 13:55:40.000000000 -0500
+++ lib/kernel/src/net_kernel.erl	2008-03-04 15:38:36.652898000 -0500
@@ -215,11 +215,12 @@
     request({connect, hidden, Node}).
 
 do_connect(Node, Type, WaitForBarred) -> %% Type = normal | hidden
+	Mode = get_dist_auto_connect(connect, Node),
     case catch ets:lookup(sys_dist, Node) of
 	{'EXIT', _} ->
 	    ?connect_failure(Node,{table_missing, sys_dist}),
 	    false;
-	[#barred_connection{}] ->
+	[#barred_connection{}] when Mode =:= once ->
 	    case WaitForBarred of
 		false ->
 		    false;
@@ -239,14 +240,14 @@
 		    end
 	    end;
 	Else ->
-	    case application:get_env(kernel, dist_auto_connect) of
-		{ok, never} ->
+	    case Mode of
+  		never ->
 		    ?connect_failure(Node,{dist_auto_connect,never}),
 		    false;
 		% This might happen due to connection close 
 		% not beeing propagated to user space yet.
 		% Save the day by just not connecting... 
-		{ok, once} when Else =/= [],
+		once when Else =/= [],
 				(hd(Else))#connection.state =:= up ->
 		    ?connect_failure(Node,{barred_connection,  
 				ets:lookup(sys_dist, Node)}),
@@ -256,6 +257,35 @@
 	    end
     end.
 
+%% @spec (Action, Node::node()) -> Mode
+%%          Action = connect | disconnect
+%%          Mode   = never | once | true
+%% @doc This function adds user-level control over the nodes we are 
+%%      allowed to connect to.  Modes `never' and `once' of 
+%%      `dist_auto_connect' are documented  in the kernel(3). A new
+%%      mode {callback, M, F} defines a function M:F/2 that should be called
+%%      on the net_kernel's connect/disconnect request.  The function's
+%%      signature is `(Action, Node) -> Mode', and its result is used
+%%      to allow/reject connection attempt.
+get_dist_auto_connect(Action, Node) ->
+    case application:get_env(kernel, dist_auto_connect) of
+    {ok, {callback, M, F}} ->
+        try M:F(Action, Node) of
+        {ok, Mode} ->
+            Mode;
+        _ ->
+            never
+        catch _:Err ->
+            error_logger:error_msg(
+                "~n** Error in dist_auto_connect callback ~w:~w/2: ~w**~n", [M,F,Err]),
+            true
+        end;
+    {ok, Other} ->
+        Other;
+    _ ->
+        true
+    end.
+
 passive_connect_monitor(Parent, Node) ->
     monitor_nodes(true,[{node_type,all}]),
     case lists:member(Node,nodes([connected])) of
@@ -849,8 +879,8 @@
     State.
 
 mark_sys_dist_nodedown(Node) ->
-    case application:get_env(kernel, dist_auto_connect) of
-	{ok, once} ->
+    case get_dist_auto_connect(disconnect, Node) of
+	once ->
 	    ets:insert(sys_dist, #barred_connection{node = Node});
 	_ ->
 	    ets:delete(sys_dist, Node)
